module version
   ! Gives the NJOY version identifier and the corresponding date.
   ! These values are updated during the NJOY revision-control process.
   implicit none
   private
   character(8),public::vers='2016.78'
   character(8),public::vday='03Feb25'
end module version
